'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import pyglet

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

pygletloader = pyglet.resource.Loader([data_dir])
textures = {}
textures["background"] = pygletloader.texture("background.png")
textures["hud"] = pygletloader.texture("hud.png")
textures["block"] = pygletloader.texture("block.png")
textures["timeup"] = pygletloader.texture("timeup.png")
textures["noobover"] = pygletloader.texture("noobover.png")
textures["newgame"] = pygletloader.texture("newgame.png")

sounds = {}
sounds["hit"] = pygletloader.media("32948__hardpcm__chip048.wav", False)
sounds["pause"] = pygletloader.media("34240__hardpcm__chip126.wav", False)
sounds["move"] = pygletloader.media("32952__hardpcm__chip052.wav", False)
sounds["rotate"] = pygletloader.media("32951__hardpcm__chip051.wav", False)
sounds["mutate"] = pygletloader.media("32949__hardpcm__chip049.wav", False)
sounds["timeup"] = pygletloader.media("34232__hardpcm__chip116.wav", False)

maintheme = pygletloader.media("maintheme.ogg")

def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)

