#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import numpy
import pyglet
from pyglet.window import key
import panel
import block
import data
from data import sounds

class status:
    FIRST = 0
    RUN = 1
    LINE = 2
    PAUSE = 3
    NOOBOVER = 4
    TIMEUP = 5

class engine:

    def __init__(self):
        self.panel = panel.panel(10, 22)
        self.messagebatch = pyglet.graphics.Batch()
        self.timeup = pyglet.sprite.Sprite(data.textures["timeup"], batch=self.messagebatch)
        self.timeup.visible = False
        self.timeup.position = (240, 64)
        self.noobover = pyglet.sprite.Sprite(data.textures["noobover"], batch=self.messagebatch)
        self.noobover.visible = False
        self.noobover.position = (240, 64)
        self.newgame = pyglet.sprite.Sprite(data.textures["newgame"], batch=self.messagebatch)
        self.newgame.visible = True
        self.newgame.position = (240, 64)
        self.textbatch = pyglet.graphics.Batch()
        self.topscore = 0
        self.texttopscore = pyglet.text.Label("0", anchor_y='top', font_size=22, batch=self.textbatch)
        self.texttopscore.color = (0, 0, 0, 255)
        self.texttopscore.y = 480-32*4
        self.texttopscore.x = 32*2
        self.score = 0
        self.textscore = pyglet.text.Label("0", anchor_y='top', font_size=22, batch=self.textbatch)
        self.textscore.x = 32*14
        self.textscore.y = 480-32*4
        self.textscore.color = (0, 0, 0, 255)
        self.seconds = 120
        self.textseconds = pyglet.text.Label("120", anchor_y='top', font_size=22, batch=self.textbatch)
        self.textseconds.color = (0, 0, 0, 255)
        self.textseconds.x = 32*3
        self.textseconds.y = 480-32*8
        self.gameover = False
        self.bg = pyglet.sprite.Sprite(data.textures["background"])
        self.bg.position = (240, 64)
        self.hud = pyglet.sprite.Sprite(data.textures["hud"])
        self.clock = pyglet.clock.Clock()
        self.clock.schedule(self.Run)
        self.clock.schedule_interval(self.UpdateSeconds, 1)
        self.staleblock_negation = []#Stack for type blocks
        self.status = status.FIRST
        self.tilebatch = pyglet.graphics.Batch()
        self.drawables = [self.bg, self.tilebatch, self.hud, self.textbatch, self.messagebatch]
        #self.drawables = [self.textscore]
        self.objecttype = numpy.dtype(object)
        self.tilesprite = numpy.empty((22, 10), self.objecttype)
        self.updatepanel = 0.0
        x = 240
        y = 64
        for i, v in numpy.ndenumerate(self.tilesprite):
            self.tilesprite[i] = pyglet.sprite.Sprite(data.textures["block"], batch=self.tilebatch)
            self.tilesprite[i].x = (240 + i[1]*16)
            self.tilesprite[i].y = 480-(64 + i[0]*16)-16
        self.mainthemeplayer = pyglet.media.Player()
        self.mainthemeplayer.eos_action = pyglet.media.Player.EOS_LOOP
        self.mainthemeplayer.queue(data.maintheme)
            
    def Run(self, time):
        self.textscore.text = str(self.score)
        panel = self.panel
        numpy_ndenumerate = numpy.ndenumerate
        tiles_panel = self.panel.tiles
        panel_blocktiles = self.panel.blocktiles
        tilesprite = self.tilesprite
        data_textures = data.textures
        self.updatepanel += time
                
        if self.status == status.RUN:
            panel.Run()
            if self.panel.CheckLines():
                self.score += self.panel.CleanLines() * 20
                self.textscore.text = str(self.score)
            #if self.panel.CheckLines():
                #self.status = engine.status.LINES
        if self.status == status.RUN:
            if panel.collision is True:
                sounds["hit"].play()
                self.score += 10 - self.staleblock_negation.count(panel.block.type) * 2
                self.textscore.text = str(self.score)
                if (len(self.staleblock_negation) == 5):
                    self.staleblock_negation.pop()
                self.staleblock_negation.insert(0, panel.block.type)
                if panel.NextBlock() is not True:
                    self.NoobOver()
            if  self.updatepanel >= 0.5:
                self.panel.block.y += 1
                self.updatepanel = 0.0
        
        #Update de sprites from panel tiles and block tiles
        for i, v in numpy_ndenumerate(self.tilesprite):
            if self.status == status.RUN:
                if panel.collision is not True and (tiles_panel[i] or panel_blocktiles[i]):
                    tilesprite[i].visible = True
                elif panel.collision is True and tiles_panel[i]:
                    tilesprite[i].visible = True
                elif panel.collision is True and panel_blocktiles[i]:
                    tilesprite[i].visible = False
                else:
                    tilesprite[i].visible = False
        #Done
        
        panel.collision = False
        
        #Update top score
        if self.score > self.topscore:
            self.topscore = self.score
            self.texttopscore.text = str(self.topscore)
    
    def UpdateSeconds(self, time):
        if self.status == status.RUN:
            self.seconds -= int(time)
            self.textseconds.text = str(self.seconds)
        if self.seconds < 10:
            self.mainthemeplayer.pitch = 1.2
        if self.seconds == 0:
            self.TimeUp()
    
    def OnKeyPress(self, symbol, modifier):
        theblock = self.panel.block
        #panel.DrawBlock(self.panel.blocktiles, block, 0)
        ltype = theblock.type
        rotation = theblock.rotation
        irotation = theblock.irotation
        if symbol == key.I or symbol == key._1:
            theblock.MutateToI()
            sounds["mutate"].play()
        elif symbol == key.O or symbol == key._2:
            theblock.MutateToO()
            sounds["mutate"].play()
        elif symbol == key.J or symbol == key._3:
            theblock.MutateToJ()
            sounds["mutate"].play()
        elif symbol == key.L or symbol == key._4:
            theblock.MutateToL()
            sounds["mutate"].play()
        elif symbol == key.S or symbol == key._5:
            theblock.MutateToS()
            sounds["mutate"].play()
        elif symbol == key.Z or symbol == key._6:
            theblock.MutateToZ()
            sounds["mutate"].play()
        elif symbol == key.T or symbol == key._7:
            theblock.MutateToT()
            sounds["mutate"].play()
        elif symbol == key.SPACE:
            theblock.Rotate()
            sounds["rotate"].play()
        elif symbol == key.LEFT:
            if self.panel.MoveBlock(-1):
                sounds["move"].play()
        elif symbol == key.RIGHT:
            if self.panel.MoveBlock(1):
                sounds["move"].play()
        elif symbol == key.DOWN:
            self.panel.DropBlock()
            return
        elif symbol == key.ENTER:
            """
            When the game is running pause it
            When the game is over start it again
            or start the game the first time
            """
            if self.status == status.RUN:
                self.status = status.PAUSE
                sounds["pause"].play()
                self.mainthemeplayer.volume = 0.5
            elif self.status == status.PAUSE:
                self.status = status.RUN
                sounds["pause"].play()
                self.mainthemeplayer.volume = 1.0
            elif self.status == status.TIMEUP or self.status == status.NOOBOVER or self.status == status.FIRST:
                #Re/start game
                self.panel.Reset()
                self.timeup.visible = False
                self.noobover.visible = False
                self.newgame.visible = False
                self.score = 0
                self.textscore.text = "0"
                self.status = status.RUN
                self.seconds = 120
                self.textseconds.text = "120"
                self.staleblock_negation = []
                self.mainthemeplayer.play()
                self.mainthemeplayer.pitch = 1.0

        if (theblock.type == block.types.I and theblock.x+1==10) or self.panel.CheckCollision():
            #If collision then revert to the last block and rotation
            theblock.type = ltype
            theblock.rotation = rotation
            theblock.irotation = irotation
            
    def TimeUp(self):
        #Blocks aren't visible
        #Shows the message "TIME UP!"
        self.status = status.TIMEUP
        self.timeup.visible = True
        self.mainthemeplayer.pause()
    
    def NoobOver(self):
        self.status = status.NOOBOVER
        self.noobover.visible = True
        self.mainthemeplayer.pause()
    
theengine = engine()
