#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import numpy
import block

def DrawBlock(tiles, block, value, yoffset=0, xoffset=0):
    """
    0, 0, 0, 0
    0, 1, 0, 0
    0, 0, 0, 0
    0, 0, 0, 0
    """
    shape = block.ReturnShape()
    y = 0
    for row in shape:
        x = 0
        for col in row:
            if shape[y, x]:
                tiles[block.y+y+yoffset-1, block.x+x+xoffset-1] = value
            x += 1
        y += 1
        #i[0] row, i[1] col

class panel:
    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.block = block.block()
        self.tiles = numpy.zeros((height, width), dtype=numpy.int)#Static draw
        self.lines = []
        #Build limits panel
        self.limits = numpy.zeros((height+2, width+2), dtype=numpy.int)
        for arow in self.limits:
            arow[0] = 1
            arow[11] = 1
        self.limits[0][:] = 1
        self.limits[23][:] = 1
        #end build panels
        #"Draw" the block in this 2 matrix
        self.blocktiles = numpy.zeros((height, width), dtype=numpy.int)#Draw the blocks
        self.blocklimits = numpy.zeros(self.limits.shape, dtype=numpy.int)
        #
        #Store collision result
        self.collisiontiles = numpy.zeros((height, width), dtype=numpy.int)
        #collision tiles se comprueba sumando tiles y blocktiles en collisiontiles
        self.collisionlimits = numpy.zeros(self.limits.shape, dtype=numpy.bool)
        #
        self.tox = 0
        self.toy = 0
        self.collision = False

    def CheckLines(self):
        y = 0
        for arow in self.tiles:
            if arow.all():
                self.lines.append(y)
            y += 1
        if len(self.lines) == 0:
            return False
        return True

    def CleanLines(self):
        nlines = 0
        for line in self.lines:
            for i in xrange(line, 0, -1):
                #self.tiles[0].fill(0)
                self.tiles[i] = self.tiles[i-1]
            nlines += 1
        self.lines = []
        return nlines

    def Reset(self):
        self.block = block.block()
        self.tiles.fill(0)
        self.collision = False

    def Run(self):
        if self.CheckCollision():
            self.collision = True
            DrawBlock(self.tiles, self.block, 1, yoffset=-1)
            return

    def NextBlock(self):
        self.block = block.block()
        if self.tiles[self.block.y, self.block.x]:
            return False
        return True

    def CheckLimit(self):
        if self.tiles[2].any():
            return True
        return False

    def CheckCollision(self):
        #First, check limits
        self.blocklimits.fill(0)
        DrawBlock(self.blocklimits, self.block, 1, 1, 1)
        self.collisionlimits.fill(False)
        numpy.equal(self.blocklimits, self.limits, self.collisionlimits)
        if self.collisionlimits[23].any():
            return True
        #Is necessary check panel left side and right side
        tcollisionlimits = self.collisionlimits.transpose()
        if tcollisionlimits[0].any():#Right side
            return True
        if tcollisionlimits[11].any():#Left side
            return True
        #Second, check tiles
        #This avoids index errors at blocktiles
        self.blocktiles.fill(0)
        DrawBlock(self.blocktiles, self.block, 1)
        self.collisiontiles.fill(0)
        numpy.add(self.blocktiles, self.tiles, self.collisiontiles)
        if self.collisiontiles.max() == 2:
            return True
        return False
        
    def MoveBlock(self, xoffset=0):
        block = self.block
        last_x = block.x
        block.x += xoffset
        if self.CheckCollision():
            block.x = last_x
            return False
        return True
        
    def DropBlock(self):
        """
        This method take the current block and increment y until collides with the tiles
        or the panel bottom
        """
        block = self.block
        CheckCollision = self.CheckCollision
        while CheckCollision() is not True:
            block.y += 1
        self.collision = True
        